require 'helper'

class Sheriff::FeatureSetTest < Test::Unit::TestCase

  context 'delicious bacon-wrapped context of chocolatey goodness' do

    setup do
      Sheriff.defined_feature_sets.clear
    end

    should 'have no defined features' do
      deny FeatureSet.all.any?
    end

    should 'be able to define a feature' do
      FeatureSet.define :share
      assert FeatureSet.all.map(&:name).include? :share
    end

    context 'with a defined feature' do
      setup do
        FeatureSet.define :share
      end

      should 'know if a feature is already defined' do
        assert FeatureSet.defined? :share
      end

      should 'know if a feature is *not* already defined' do
        deny FeatureSet.defined? :undefined_feature
      end

      should 'raise when redefining the same feature' do
        assert_raise Sheriff::FeatureSet::AlreadyDefinedError do
          FeatureSet.define :share
        end
      end

      should 'be able to find a feature' do
        feature = FeatureSet(:share)
        assert_equal :share, feature.name
      end

    end

    context 'when executing within a context' do

      setup do
        class FakeContext
          # include Sheriff::ContextInitializer

          def show_method
            feature_set(:fake_feature).visible?
          end
          def hide_method
            feature_set(:fake_feature).visible?
          end
        end
        FeatureSet.define :fake_feature do |f|
          f.show :show_method, FakeContext
          f.hide :hide_method, FakeContext
        end
      end

      should 'be visible for FakeContext#show' do
        assert FakeContext.new.show_method
      end

      should 'be hidden for FakeContext#hide' do
        deny FakeContext.new.hide_method
      end
    end

  end
end
